/*
 * ksi_prim.h
 * primitive functions
 *
 * Copyright (C) 1997-2000, Ivan Demakov.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without a written agreement
 * is hereby granted, provided that the above copyright notice and this
 * paragraph and the following two paragraphs appear in all copies.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE AUTHORS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *
 * Author:        Ivan Demakov <demakov@users.sourceforge.net>
 * Creation date: Thu Nov 27 03:24:35 1997
 * Last Update:   Mon Mar 20 22:59:21 2000
 *
 * 
 * $Id: ksi_prim.h,v 1.1.1.1 2000/07/25 19:13:30 demakov Exp $
 *
 */

#ifndef KSI_PRIM_H
#define KSI_PRIM_H


#define KSI_CALL_ARG0		((ksi_prim_call) 0)
#define KSI_CALL_ARG1		((ksi_prim_call) 1)
#define KSI_CALL_ARG2		((ksi_prim_call) 2)
#define KSI_CALL_ARG3		((ksi_prim_call) 3)
#define KSI_CALL_ARG4		((ksi_prim_call) 4)
#define KSI_CALL_ARG5		((ksi_prim_call) 5)
#define KSI_CALL_ARG6		((ksi_prim_call) 6)
#define KSI_CALL_ARG7		((ksi_prim_call) 7)
#define KSI_CALL_ARG8		((ksi_prim_call) 8)
#define KSI_CALL_ARG9		((ksi_prim_call) 9)
#define KSI_CALL_VECT		((ksi_prim_call) 10)
#define KSI_CALL_MAIN		((ksi_prim_call) 11)
#define KSI_CALL_RETN		((ksi_prim_call) 12)

#define KSI_CALL_LAST_COD	12


typedef ksi_obj (*ksi_prim_call) (void* data, int argc, ksi_obj* argv);


struct Ksi_Prim_Def
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  const char	*name;	/* name of primitive */
  void		*data;	/* call data */
  ksi_prim_call	call;	/* caller */
  int		reqv;	/* number of required args */
};


struct Ksi_Prim
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  unsigned		itag;
  ksi_prim_call		call;
  void*			data;
  const char*		name;
};

#define KSI_PRIM_CALL(x)	(((ksi_prim) (x)) -> call)
#define KSI_PRIM_DATA(x)	(((ksi_prim) (x)) -> data)
#define KSI_PRIM_NAME(x)	(((ksi_prim) (x)) -> name)


#ifdef __cplusplus
extern "C" {
#endif

SI_API
ksi_obj
ksi_make_prim (ksi_prim_call call, void* data, const char* name);

SI_API
void
ksi_reg_unit (struct Ksi_Prim_Def* prims);

SI_API
ksi_obj
ksi_reg_prim (struct Ksi_Prim_Def* prim);

SI_API
ksi_obj
ksi_defun (const char* name, void* fun, ksi_prim_call call, int reqv);


#ifdef __cplusplus
}
#endif

#endif

 /* End of file */
